@@ -72,6 +72,9 @@ struct _GstSchroEnc
   uint64_t granulepos_offset;
   uint64_t granulepos_low;
   uint64_t granulepos_hi;
+  gboolean started;
+  gint64 timestamp_offset;
+  int picture_number;
 
   SchroEncoder *encoder;
   SchroVideoFormat *video_format;
@@ -90,17 +93,7 @@ enum
 
 enum
 {
-  ARG_0,
-  ARG_ENGINE,
-  ARG_REF_DISTANCE,
-  ARG_TRANSFORM_DEPTH,
-  ARG_INTRA_WAVELET,
-  ARG_INTER_WAVELET,
-  ARG_QUANT_BASE,
-  ARG_QUANT_OFFSET_NONREF,
-  ARG_QUANT_OFFSET_SUBBAND,
-  ARG_QUANT_DC,
-  ARG_QUANT_DC_OFFSET_NONREF
+  ARG_0
 };
 
 static void gst_schro_enc_finalize (GObject *object);
@@ -122,7 +115,7 @@ static GstStaticPadTemplate gst_schro_enc_sink_template =
     GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS (GST_VIDEO_CAPS_YUV ("{ I420, YUY2, AYUV }"))
+    GST_STATIC_CAPS (GST_VIDEO_CAPS_YUV ("{ I420, YV12, YUY2, UYVY, AYUV }"))
     );
 
 static GstStaticPadTemplate gst_schro_enc_src_template =
@@ -157,12 +150,7 @@ gst_schro_enc_class_init (GstSchroEncClass * klass)
 {
   GObjectClass *gobject_class;
   GstElementClass *gstelement_class;
-  static const char *arg_names[] = {
-    "engine", "ref_distance", "transform_depth", "intra_wavelet",
-    "inter_wavelet", "quant_base", "quant_offset_nonref",
-    "quant_offset_subband", "quant_dc", "quant_dc_offset_nonref" };
   int i;
-  SchroEncoder *enc;
 
   gobject_class = G_OBJECT_CLASS (klass);
   gstelement_class = GST_ELEMENT_CLASS (klass);
@@ -171,17 +159,39 @@ gst_schro_enc_class_init (GstSchroEncClass * klass)
   gobject_class->get_property = gst_schro_enc_get_property;
   gobject_class->finalize = gst_schro_enc_finalize;
 
-  enc = schro_encoder_new ();
-  for(i=0;i<10;i++){
-    int min, max, val;
-    schro_encoder_preference_get_range (enc, i, &min, &max);
-    val = schro_encoder_preference_get (enc, i);
-    g_object_class_install_property (gobject_class, ARG_ENGINE + i,
-        g_param_spec_int (arg_names[i], arg_names[i], arg_names[i],
-          min, max, val, G_PARAM_READWRITE));
-  }
+  for(i=0;i<schro_encoder_get_n_settings();i++){
+    const SchroEncoderSetting *setting;
 
-  schro_encoder_free (enc);
+    setting = schro_encoder_get_setting_info (i);
+
+    switch (setting->type) {
+      case SCHRO_ENCODER_SETTING_TYPE_BOOLEAN:
+        g_object_class_install_property (gobject_class, i + 1,
+            g_param_spec_boolean (setting->name, setting->name, setting->name,
+              setting->default_value, G_PARAM_READWRITE));
+        break;
+      case SCHRO_ENCODER_SETTING_TYPE_INT:
+        g_object_class_install_property (gobject_class, i + 1,
+            g_param_spec_int (setting->name, setting->name, setting->name,
+              setting->min, setting->max, setting->default_value,
+              G_PARAM_READWRITE));
+        break;
+      case SCHRO_ENCODER_SETTING_TYPE_ENUM:
+        g_object_class_install_property (gobject_class, i + 1,
+            g_param_spec_int (setting->name, setting->name, setting->name,
+              setting->min, setting->max, setting->default_value,
+              G_PARAM_READWRITE));
+        break;
+      case SCHRO_ENCODER_SETTING_TYPE_DOUBLE:
+        g_object_class_install_property (gobject_class, i + 1,
+            g_param_spec_double (setting->name, setting->name, setting->name,
+              setting->min, setting->max, setting->default_value,
+              G_PARAM_READWRITE));
+        break;
+      default:
+        break;
+    }
+  }
 
   gstelement_class->change_state = gst_schro_enc_change_state;
 }
@@ -226,15 +236,16 @@ gst_schro_enc_sink_setcaps (GstPad *pad, GstCaps *caps)
   gst_structure_get_fraction (structure, "pixel-aspect-ratio",
       &schro_enc->par_n, &schro_enc->par_d);
 
-  /* SD480 has most of the defaults that GStreamer assumes */
-  schro_params_set_video_format (schro_enc->video_format,
-      SCHRO_VIDEO_FORMAT_SD480);
+  schro_video_format_set_std_video_format (schro_enc->video_format,
+      SCHRO_VIDEO_FORMAT_CUSTOM);
 
   switch (schro_enc->fourcc) {
     case GST_MAKE_FOURCC('I','4','2','0'):
+    case GST_MAKE_FOURCC('Y','V','1','2'):
       schro_enc->video_format->chroma_format = SCHRO_CHROMA_420;
       break;
     case GST_MAKE_FOURCC('Y','U','Y','2'):
+    case GST_MAKE_FOURCC('U','Y','V','Y'):
       schro_enc->video_format->chroma_format = SCHRO_CHROMA_422;
       break;
     case GST_MAKE_FOURCC('A','Y','U','V'):
@@ -255,6 +266,11 @@ gst_schro_enc_sink_setcaps (GstPad *pad, GstCaps *caps)
   schro_enc->video_format->aspect_ratio_numerator = schro_enc->par_n;
   schro_enc->video_format->aspect_ratio_denominator = schro_enc->par_d;
 
+  schro_video_format_set_std_signal_range (schro_enc->video_format,
+      SCHRO_SIGNAL_RANGE_8BIT_VIDEO);
+  schro_video_format_set_std_colour_spec (schro_enc->video_format,
+      SCHRO_COLOUR_SPEC_HDTV);
+
   schro_encoder_set_video_format (schro_enc->encoder, schro_enc->video_format);
 
   schro_enc->duration = gst_util_uint64_scale_int (GST_SECOND,
@@ -291,22 +307,24 @@ gst_schro_enc_set_property (GObject * object, guint prop_id,
   src = GST_SCHRO_ENC (object);
 
   GST_DEBUG ("gst_schro_enc_set_property");
-  switch (prop_id) {
-    case ARG_ENGINE:
-    case ARG_REF_DISTANCE:
-    case ARG_TRANSFORM_DEPTH:
-    case ARG_INTRA_WAVELET:
-    case ARG_INTER_WAVELET:
-    case ARG_QUANT_BASE:
-    case ARG_QUANT_OFFSET_NONREF:
-    case ARG_QUANT_OFFSET_SUBBAND:
-    case ARG_QUANT_DC:
-    case ARG_QUANT_DC_OFFSET_NONREF:
-      schro_encoder_preference_set (src->encoder, prop_id - ARG_ENGINE,
-          g_value_get_int(value));
-      break;
-    default:
-      break;
+
+  if (prop_id >= 1) {
+    const SchroEncoderSetting *setting;
+    setting = schro_encoder_get_setting_info (prop_id - 1);
+    switch(G_VALUE_TYPE(value)) {
+      case G_TYPE_DOUBLE:
+        schro_encoder_setting_set_double (src->encoder, setting->name,
+            g_value_get_double(value));
+        break;
+      case G_TYPE_INT:
+        schro_encoder_setting_set_double (src->encoder, setting->name,
+            g_value_get_int(value));
+        break;
+      case G_TYPE_BOOLEAN:
+        schro_encoder_setting_set_double (src->encoder, setting->name,
+            g_value_get_boolean(value));
+        break;
+    }
   }
 }
 
@@ -319,23 +337,23 @@ gst_schro_enc_get_property (GObject * object, guint prop_id, GValue * value,
   g_return_if_fail (GST_IS_SCHRO_ENC (object));
   src = GST_SCHRO_ENC (object);
 
-  switch (prop_id) {
-    case ARG_ENGINE:
-    case ARG_REF_DISTANCE:
-    case ARG_TRANSFORM_DEPTH:
-    case ARG_INTRA_WAVELET:
-    case ARG_INTER_WAVELET:
-    case ARG_QUANT_BASE:
-    case ARG_QUANT_OFFSET_NONREF:
-    case ARG_QUANT_OFFSET_SUBBAND:
-    case ARG_QUANT_DC:
-    case ARG_QUANT_DC_OFFSET_NONREF:
-      g_value_set_int (value,
-          schro_encoder_preference_get (src->encoder, prop_id - ARG_ENGINE));
-      break;
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
+  if (prop_id >= 1) {
+    const SchroEncoderSetting *setting;
+    setting = schro_encoder_get_setting_info (prop_id - 1);
+    switch(G_VALUE_TYPE(value)) {
+      case G_TYPE_DOUBLE:
+        g_value_set_double (value,
+            schro_encoder_setting_get_double (src->encoder, setting->name));
+        break;
+      case G_TYPE_INT:
+        g_value_set_int (value,
+            schro_encoder_setting_get_double (src->encoder, setting->name));
+        break;
+      case G_TYPE_BOOLEAN:
+        g_value_set_boolean (value,
+            schro_encoder_setting_get_double (src->encoder, setting->name));
+        break;
+    }
   }
 }
 
@@ -397,9 +415,15 @@ gst_schro_buffer_wrap (GstSchroEnc *schro_enc, GstBuffer *buf,
     case GST_MAKE_FOURCC('I','4','2','0'):
       frame = schro_frame_new_from_data_I420 (GST_BUFFER_DATA (buf), width, height);
       break;
+    case GST_MAKE_FOURCC('Y','V','1','2'):
+      frame = schro_frame_new_from_data_YV12 (GST_BUFFER_DATA (buf), width, height);
+      break;
     case GST_MAKE_FOURCC('Y','U','Y','2'):
       frame = schro_frame_new_from_data_YUY2 (GST_BUFFER_DATA (buf), width, height);
       break;
+    case GST_MAKE_FOURCC('U','Y','V','Y'):
+      frame = schro_frame_new_from_data_UYVY (GST_BUFFER_DATA (buf), width, height);
+      break;
     case GST_MAKE_FOURCC('A','Y','U','V'):
       frame = schro_frame_new_from_data_AYUV (GST_BUFFER_DATA (buf), width, height);
       break;
@@ -412,7 +436,7 @@ gst_schro_buffer_wrap (GstSchroEnc *schro_enc, GstBuffer *buf,
 }
 
 #define OGG_DIRAC_GRANULE_SHIFT 30
-#define OGG_DIRAC_GRANULE_LOW_MASK ((1<<OGG_DIRAC_GRANULE_SHIFT)-1)
+#define OGG_DIRAC_GRANULE_LOW_MASK ((1ULL<<OGG_DIRAC_GRANULE_SHIFT)-1)
 
 static gint64
 granulepos_to_frame (gint64 granulepos)
@@ -575,6 +599,22 @@ error:
   return res;
 }
 
+static gboolean
+gst_pad_is_negotiated (GstPad *pad)
+{
+  GstCaps *caps;
+
+  g_return_val_if_fail (pad != NULL, FALSE);
+
+  caps = gst_pad_get_negotiated_caps (pad);
+  if (caps) {
+    gst_caps_unref (caps);
+    return TRUE;
+  }
+
+  return FALSE;
+}
+
 static GstFlowReturn
 gst_schro_enc_chain (GstPad *pad, GstBuffer *buf)
 {
@@ -584,6 +624,9 @@ gst_schro_enc_chain (GstPad *pad, GstBuffer *buf)
 
   schro_enc = GST_SCHRO_ENC (gst_pad_get_parent (pad));
 
+  if (!gst_pad_is_negotiated (pad)) {
+    return GST_FLOW_NOT_NEGOTIATED;
+  }
   if (GST_BUFFER_TIMESTAMP (buf) < schro_enc->segment_start) {
     GST_DEBUG("dropping early buffer");
     return GST_FLOW_OK;
@@ -596,6 +639,13 @@ gst_schro_enc_chain (GstPad *pad, GstBuffer *buf)
     GST_DEBUG("using granulepos offset %lld", schro_enc->granulepos_offset);
     schro_enc->granulepos_hi = 0;
     schro_enc->got_offset = TRUE;
+
+    schro_enc->timestamp_offset = GST_BUFFER_TIMESTAMP(buf);
+    schro_enc->picture_number = 0;
+  }
+  if (!schro_enc->started) {
+    schro_encoder_start (schro_enc->encoder);
+    schro_enc->started = TRUE;
   }
 
   frame = gst_schro_buffer_wrap (schro_enc, buf, schro_enc->width, schro_enc->height);
@@ -617,9 +667,10 @@ gst_schro_enc_process (GstSchroEnc *schro_enc)
   GstBuffer *outbuf;
   GstFlowReturn ret;
   int presentation_frame;
+  int parse_code;
 
   while (1) {
-    switch (schro_encoder_iterate (schro_enc->encoder)) {
+    switch (schro_encoder_wait (schro_enc->encoder)) {
       case SCHRO_STATE_NEED_FRAME:
       case SCHRO_STATE_END_OF_STREAM:
         return GST_FLOW_OK;
@@ -631,8 +682,9 @@ gst_schro_enc_process (GstSchroEnc *schro_enc)
           /* FIXME This shouldn't happen */
           return GST_FLOW_ERROR;
         }
+        parse_code = encoded_buffer->data[4];
 
-        if (schro_decoder_is_access_unit (encoded_buffer)) {
+        if (SCHRO_PARSE_CODE_IS_SEQ_HEADER(parse_code)) {
           schro_enc->granulepos_hi = schro_enc->granulepos_offset +
             presentation_frame + 1;
         }
@@ -643,35 +695,49 @@ gst_schro_enc_process (GstSchroEnc *schro_enc)
         outbuf = gst_buffer_new_and_alloc (encoded_buffer->length);
         memcpy (GST_BUFFER_DATA (outbuf), encoded_buffer->data,
             encoded_buffer->length);
-        gst_buffer_set_caps (outbuf, gst_pad_get_caps(schro_enc->srcpad));
+        gst_buffer_set_caps (outbuf,
+            gst_caps_new_simple ("video/x-dirac",
+              "width", G_TYPE_INT, schro_enc->width,
+              "height", G_TYPE_INT, schro_enc->height,
+              "framerate", GST_TYPE_FRACTION, schro_enc->fps_n,
+              schro_enc->fps_d,
+              NULL));
 
-        GST_BUFFER_OFFSET_END (outbuf) =
-          (schro_enc->granulepos_hi<<30) + schro_enc->granulepos_low;
-        GST_BUFFER_OFFSET (outbuf) = gst_util_uint64_scale (
-            (schro_enc->granulepos_hi + schro_enc->granulepos_low),
-            schro_enc->fps_d * GST_SECOND, schro_enc->fps_n);
-
-        GST_BUFFER_TIMESTAMP (outbuf) = gst_util_uint64_scale (
-            (schro_enc->granulepos_hi + schro_enc->granulepos_low),
-            schro_enc->fps_d * GST_SECOND, schro_enc->fps_n);
-        if (schro_decoder_is_picture (encoded_buffer)) {
+        if (SCHRO_PARSE_CODE_IS_PICTURE(parse_code)) {
+          GST_BUFFER_OFFSET_END (outbuf) =
+            (schro_enc->granulepos_hi<<OGG_DIRAC_GRANULE_SHIFT) +
+            schro_enc->granulepos_low;
+          GST_BUFFER_OFFSET (outbuf) = gst_util_uint64_scale (
+              (schro_enc->granulepos_hi + schro_enc->granulepos_low),
+              schro_enc->fps_d * GST_SECOND, schro_enc->fps_n);
           GST_BUFFER_DURATION (outbuf) = schro_enc->duration;
+          GST_BUFFER_TIMESTAMP (outbuf) = 
+            schro_enc->timestamp_offset + gst_util_uint64_scale (
+              schro_enc->picture_number,
+              schro_enc->fps_d * GST_SECOND, schro_enc->fps_n);
+          schro_enc->picture_number++;
+          if (!SCHRO_PARSE_CODE_IS_INTRA(parse_code)) {
+            GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_DELTA_UNIT);
+          }
         } else {
-          GST_BUFFER_DURATION (outbuf) = 0;
+          GST_BUFFER_OFFSET_END (outbuf) = 0;
+          GST_BUFFER_OFFSET (outbuf) = 0;
+          GST_BUFFER_DURATION (outbuf) = -1;
+          //GST_BUFFER_TIMESTAMP (outbuf) = -1;
+          GST_BUFFER_TIMESTAMP (outbuf) = 
+            schro_enc->timestamp_offset + gst_util_uint64_scale (
+              schro_enc->picture_number,
+              schro_enc->fps_d * GST_SECOND, schro_enc->fps_n);
         }
 
         GST_INFO("size %d offset %lld granulepos %llu:%llu timestamp %lld duration %lld",
             GST_BUFFER_SIZE (outbuf),
             GST_BUFFER_OFFSET (outbuf),
-            GST_BUFFER_OFFSET_END (outbuf)>>30,
-            GST_BUFFER_OFFSET_END (outbuf)&((1<<30) - 1),
+            GST_BUFFER_OFFSET_END (outbuf)>>OGG_DIRAC_GRANULE_SHIFT,
+            GST_BUFFER_OFFSET_END (outbuf)&OGG_DIRAC_GRANULE_LOW_MASK,
             GST_BUFFER_TIMESTAMP (outbuf),
             GST_BUFFER_DURATION (outbuf));
 
-        if (schro_decoder_is_intra (encoded_buffer)) {
-          GST_BUFFER_FLAG_UNSET (outbuf, GST_BUFFER_FLAG_DELTA_UNIT);
-        }
-
         schro_buffer_unref (encoded_buffer);
         
         ret = gst_pad_push (schro_enc->srcpad, outbuf);
