@@ -5,8 +5,11 @@
 #include <schroedinger/schro.h>
 #include <liboil/liboil.h>
 #include <stdlib.h>
+#include <schroedinger/schrocuda.h>
 
 extern int _schro_decode_prediction_only;
+extern int _schro_dump_enable;
+extern int _schro_motion_ref;
 
 /**
  * schro_init:
@@ -18,6 +21,11 @@ void
 schro_init(void)
 {
   const char *s;
+  static int inited = FALSE;
+
+  if (inited) return;
+
+  inited = TRUE;
 
   oil_init();
 
@@ -36,5 +44,19 @@ schro_init(void)
   if (s && s[0]) {
     _schro_decode_prediction_only = TRUE;
   }
+
+  s = getenv ("SCHRO_MOTION_REF");
+  if (s && s[0]) {
+    _schro_motion_ref = TRUE;
+  }
+
+  s = getenv ("SCHRO_DUMP");
+  if (s && s[0]) {
+    _schro_dump_enable = TRUE;
+  }
+
+#ifdef HAVE_CUDA
+  schro_cuda_init ();
+#endif
 }
 
