@@ -10,6 +10,7 @@ typedef struct _SchroBuffer SchroBuffer;
 
 struct _SchroBuffer
 {
+  /*< private >*/
   unsigned char *data;
   int length;
 
@@ -21,33 +22,15 @@ struct _SchroBuffer
   void *priv;
 };
 
-#if 0
-struct _SchroBufferQueue
-{
-  GList *buffers;
-  int depth;
-  int offset;
-};
-#endif
-
 SchroBuffer *schro_buffer_new (void);
 SchroBuffer *schro_buffer_new_and_alloc (int size);
 SchroBuffer *schro_buffer_new_with_data (void *data, int size);
 SchroBuffer *schro_buffer_new_subbuffer (SchroBuffer * buffer, int offset,
     int length);
+SchroBuffer *schro_buffer_dup (SchroBuffer * buffer);
 SchroBuffer * schro_buffer_ref (SchroBuffer * buffer);
 void schro_buffer_unref (SchroBuffer * buffer);
-
-#if 0
-SchroBufferQueue *schro_buffer_queue_new (void);
-void schro_buffer_queue_free (SchroBufferQueue * queue);
-int schro_buffer_queue_get_depth (SchroBufferQueue * queue);
-int schro_buffer_queue_get_offset (SchroBufferQueue * queue);
-void schro_buffer_queue_push (SchroBufferQueue * queue,
-    SchroBuffer * buffer);
-SchroBuffer *schro_buffer_queue_pull (SchroBufferQueue * queue, int len);
-SchroBuffer *schro_buffer_queue_peek (SchroBufferQueue * queue, int len);
-#endif
+int schro_buffer_compare_data (SchroBuffer *a, SchroBuffer *b);
 
 SCHRO_END_DECLS
 
