@@ -17,6 +17,22 @@ enum
   SCHRO_LEVEL_LOG
 };
 
+#ifdef SCHRO_ENABLE_UNSTABLE_API
+enum
+{
+  SCHRO_DUMP_SUBBAND_CURVE,
+  SCHRO_DUMP_SUBBAND_EST,
+  SCHRO_DUMP_PICTURE,
+  SCHRO_DUMP_PSNR,
+  SCHRO_DUMP_SSIM,
+  SCHRO_DUMP_LAMBDA_CURVE,
+  SCHRO_DUMP_HIST_TEST,
+  SCHRO_DUMP_SCENE_CHANGE,
+  SCHRO_DUMP_PHASE_CORR,
+  SCHRO_DUMP_LAST
+};
+#endif
+
 #define SCHRO_ERROR(...) \
   SCHRO_DEBUG_LEVEL(SCHRO_LEVEL_ERROR, __VA_ARGS__)
 #define SCHRO_WARNING(...) \
@@ -50,6 +66,12 @@ void schro_debug_log (int level, const char *file, const char *function,
 void schro_debug_set_level (int level);
 int schro_debug_get_level (void);
 
+#ifdef SCHRO_ENABLE_UNSTABLE_API
+
+void schro_dump (int type, const char *format, ...);
+
+#endif
+
 SCHRO_END_DECLS
 
 #endif
