@@ -2,9 +2,7 @@
 #ifndef __SCHRO_QUEUE_H__
 #define __SCHRO_QUEUE_H__
 
-#include <schroedinger/schroparams.h>
 #include <schroedinger/schroutils.h>
-#include <schroedinger/schro-stdint.h>
 
 SCHRO_BEGIN_DECLS
 
@@ -26,6 +24,7 @@ struct _SchroQueue {
   SchroQueueFreeFunc free;
 };
 
+#ifdef SCHRO_ENABLE_UNSTABLE_API
 
 SchroQueue *schro_queue_new (int size, SchroQueueFreeFunc func);
 void schro_queue_free (SchroQueue *queue);
@@ -41,6 +40,8 @@ void * schro_queue_pull (SchroQueue *queue);
 int schro_queue_is_full (SchroQueue *queue);
 int schro_queue_is_empty (SchroQueue *queue);
 
+#endif
+
 SCHRO_END_DECLS
 
 #endif
