@@ -24,7 +24,7 @@ void
 schro_split_ext_desl93 (int16_t *hi, int16_t *lo, int n)
 {
   static const int16_t stage1_weights[] = { 1, -9, -9, 1 };
-  static const int16_t stage2_weights[] = { 1, 1 };
+  //static const int16_t stage2_weights[] = { 1, 1 };
   static const int16_t stage1_offset_shift[] = { 7, 4 };
   static const int16_t stage2_offset_shift[] = { 2, 2 };
 
@@ -36,26 +36,29 @@ schro_split_ext_desl93 (int16_t *hi, int16_t *lo, int n)
 
   lo[-1] = lo[0];
 
-  oil_mas2_add_s16 (hi, hi, lo - 1, stage2_weights, stage2_offset_shift, n);
+  //oil_mas2_add_s16 (hi, hi, lo - 1, stage2_weights, stage2_offset_shift, n);
+  oil_add2_rshift_add_s16 (hi, hi, lo - 1, lo, stage2_offset_shift, n);
 }
 
 void
 schro_split_ext_53 (int16_t *hi, int16_t *lo, int n)
 {
-  static const int16_t stage1_weights[] = { -1, -1 };
-  static const int16_t stage2_weights[] = { 1, 1 };
-  static const int16_t stage1_offset_shift[] = { 0, 1 };
+  //static const int16_t stage1_weights[] = { -1, -1 };
+  //static const int16_t stage2_weights[] = { 1, 1 };
+  static const int16_t stage1_offset_shift[] = { 1, 1 };
   static const int16_t stage2_offset_shift[] = { 2, 2 };
 
   hi[-1] = hi[0];
   hi[n] = hi[n-1];
 
-  oil_mas2_add_s16 (lo, lo, hi, stage1_weights, stage1_offset_shift, n);
+  //oil_mas2_add_s16 (lo, lo, hi, stage1_weights, stage1_offset_shift, n);
+  oil_add2_rshift_sub_s16 (lo, lo, hi, hi+1, stage1_offset_shift, n);
 
   lo[-1] = lo[0];
   lo[n] = lo[n-1];
 
-  oil_mas2_add_s16 (hi, hi, lo - 1, stage2_weights, stage2_offset_shift, n);
+  //oil_mas2_add_s16 (hi, hi, lo - 1, stage2_weights, stage2_offset_shift, n);
+  oil_add2_rshift_add_s16 (hi, hi, lo - 1, lo, stage2_offset_shift, n);
 }
 
 void
@@ -156,9 +159,9 @@ schro_split_ext_daub97 (int16_t *hi, int16_t *lo, int n)
 void
 schro_synth_ext_desl93 (int16_t *hi, int16_t *lo, int n)
 {
-  static const int16_t stage1_weights[] = { -1, -1 };
+  //static const int16_t stage1_weights[] = { -1, -1 };
   static const int16_t stage2_weights[] = { -1, 9, 9, -1 };
-  static const int16_t stage1_offset_shift[] = { 1, 2 };
+  static const int16_t stage1_offset_shift[] = { 2, 2 };
   static const int16_t stage2_offset_shift[] = { 8, 4 };
 
   lo[-2] = lo[0];
@@ -166,7 +169,8 @@ schro_synth_ext_desl93 (int16_t *hi, int16_t *lo, int n)
   lo[n] = lo[n-1];
   lo[n+1] = lo[n-1];
 
-  oil_mas2_add_s16 (hi, hi, lo - 1, stage1_weights, stage1_offset_shift, n);
+  //oil_mas2_add_s16 (hi, hi, lo - 1, stage1_weights, stage1_offset_shift, n);
+  oil_add2_rshift_sub_s16 (hi, hi, lo - 1, lo, stage1_offset_shift, n);
 
   hi[-2] = hi[0];
   hi[-1] = hi[0];
@@ -179,20 +183,22 @@ schro_synth_ext_desl93 (int16_t *hi, int16_t *lo, int n)
 void
 schro_synth_ext_53 (int16_t *hi, int16_t *lo, int n)
 {
-  static const int16_t stage1_weights[] = { -1, -1 };
-  static const int16_t stage2_weights[] = { 1, 1 };
-  static const int16_t stage1_offset_shift[] = { 1, 2 };
+  //static const int16_t stage1_weights[] = { -1, -1 };
+  //static const int16_t stage2_weights[] = { 1, 1 };
+  static const int16_t stage1_offset_shift[] = { 2, 2 };
   static const int16_t stage2_offset_shift[] = { 1, 1 };
 
   lo[-1] = lo[0];
   lo[n] = lo[n-1];
 
-  oil_mas2_add_s16 (hi, hi, lo - 1, stage1_weights, stage1_offset_shift, n);
+  //oil_mas2_add_s16 (hi, hi, lo - 1, stage1_weights, stage1_offset_shift, n);
+  oil_add2_rshift_sub_s16 (hi, hi, lo - 1, lo, stage1_offset_shift, n);
 
   hi[-1] = hi[0];
   hi[n] = hi[n-1];
 
-  oil_mas2_add_s16 (lo, lo, hi, stage2_weights, stage2_offset_shift, n);
+  //oil_mas2_add_s16 (lo, lo, hi, stage2_weights, stage2_offset_shift, n);
+  oil_add2_rshift_add_s16 (lo, lo, hi, hi+1, stage2_offset_shift, n);
 }
 
 void
@@ -296,6 +302,9 @@ void schro_iwt_desl_9_3 (int16_t *data, int stride, int width, int height,
 
 #define ROW(row) ((int16_t *)OFFSET(data, (row)*stride))
 
+  /* FIXME */
+  SCHRO_ASSERT(height>=6);
+
   for(i=0;i<height + 6;i++){
     int i1 = i-4;
     int i2 = i-6;
@@ -339,17 +348,29 @@ void schro_iwt_desl_9_3 (int16_t *data, int stride, int width, int height,
     if ((i2&1) == 0 && i2>=0 && i2 < height) {
       static const int16_t stage2_offset_shift[] = { 2, 2 };
       if (i2 == 0) {
-        static const int16_t stage2_weights[] = { 1, 1 };
+        //static const int16_t stage2_weights[] = { 1, 1 };
+        oil_add2_rshift_add_s16 (
+            OFFSET(data,i2*stride), OFFSET(data, i2*stride),
+            OFFSET(data, (i2+1)*stride), OFFSET(data, (i2+1)*stride),
+            stage2_offset_shift, width);
+#if 0
         oil_mas2_across_add_s16 (
             OFFSET(data,i2*stride), OFFSET(data, i2*stride),
             OFFSET(data, (i2+1)*stride), OFFSET(data, (i2+1)*stride),
             stage2_weights, stage2_offset_shift, width);
+#endif
       } else {
-        static const int16_t stage2_weights[] = { 1, 1 };
+        //static const int16_t stage2_weights[] = { 1, 1 };
+        oil_add2_rshift_add_s16 (
+            OFFSET(data,i2*stride), OFFSET(data, i2*stride),
+            OFFSET(data, (i2-1)*stride), OFFSET(data, (i2+1)*stride),
+            stage2_offset_shift, width);
+#if 0
         oil_mas2_across_add_s16 (
             OFFSET(data,i2*stride), OFFSET(data, i2*stride),
             OFFSET(data, (i2-1)*stride), OFFSET(data, (i2+1)*stride),
             stage2_weights, stage2_offset_shift, width);
+#endif
       }
     }
   }
@@ -358,9 +379,9 @@ void schro_iwt_desl_9_3 (int16_t *data, int stride, int width, int height,
 void schro_iwt_5_3 (int16_t *data, int stride, int width, int height,
     int16_t *tmp)
 {
-  static const int16_t stage1_weights[] = { -1, -1 };
-  static const int16_t stage2_weights[] = { 1, 1 };
-  static const int16_t stage1_offset_shift[] = { 0, 1 };
+  //static const int16_t stage1_weights[] = { -1, -1 };
+  //static const int16_t stage2_weights[] = { 1, 1 };
+  static const int16_t stage1_offset_shift[] = { 1, 1 };
   static const int16_t stage2_offset_shift[] = { 2, 2 };
   int i;
   int16_t one = 1;
@@ -384,24 +405,40 @@ void schro_iwt_5_3 (int16_t *data, int stride, int width, int height,
       } else {
         d = OFFSET(data,(height-2)*stride);
       }
+      oil_add2_rshift_sub_s16 (
+          OFFSET(data, (i-1)*stride),
+          OFFSET(data, (i-1)*stride),
+          OFFSET(data, (i-2)*stride),
+          d,
+          stage1_offset_shift, width);
+#if 0
       oil_mas2_across_add_s16 (
           OFFSET(data, (i-1)*stride),
           OFFSET(data, (i-1)*stride),
           OFFSET(data, (i-2)*stride),
           d,
           stage1_weights, stage1_offset_shift, width);
+#endif
 
       if (i-3>=0) {
         d = OFFSET(data, (i-3)*stride);
       } else {
         d = OFFSET(data, 1*stride);
       }
+      oil_add2_rshift_add_s16 (
+          OFFSET(data, (i-2)*stride),
+          OFFSET(data, (i-2)*stride),
+          d,
+          OFFSET(data, (i-1)*stride),
+          stage2_offset_shift, width);
+#if 0
       oil_mas2_across_add_s16 (
           OFFSET(data, (i-2)*stride),
           OFFSET(data, (i-2)*stride),
           d,
           OFFSET(data, (i-1)*stride),
           stage2_weights, stage2_offset_shift, width);
+#endif
     }
   }
 #undef ROW
@@ -413,6 +450,9 @@ void schro_iwt_13_5 (int16_t *data, int stride, int width, int height,
   int i;
   int16_t one = 1;
 
+  /* FIXME */
+  SCHRO_ASSERT(height>=6);
+
 #define ROW(row) ((int16_t *)OFFSET(data, (row)*stride))
   for(i=0;i<height + 8;i++){
     int i1 = i-4;
@@ -523,16 +563,14 @@ void schro_iwt_haar1 (int16_t *data, int stride, int width, int height, int16_t
   schro_iwt_haar (data, stride, width, height, tmp, 1);
 }
 
-void schro_iwt_haar2 (int16_t *data, int stride, int width, int height, int16_t *tmp)
-{
-  schro_iwt_haar (data, stride, width, height, tmp, 2);
-}
-
 void schro_iwt_fidelity (int16_t *data, int stride, int width, int height,
     int16_t *tmp)
 {
   int i;
 
+  /* FIXME */
+  SCHRO_ASSERT(height>=16);
+
 #define ROW(row) ((int16_t *)OFFSET(data, (row)*stride))
   for(i=0;i<height + 16;i++){
     int i1 = i-8;
@@ -674,33 +712,32 @@ void schro_iwt_daub_9_7 (int16_t *data, int stride, int width, int height,
 }
 
 void
-schro_wavelet_transform_2d (int filter, int16_t *data, int stride, int width,
-    int height, int16_t *tmp)
+schro_wavelet_transform_2d (SchroFrameData *fd, int filter, int16_t *tmp)
 {
+  SCHRO_ASSERT(SCHRO_FRAME_FORMAT_DEPTH(fd->format) ==
+      SCHRO_FRAME_FORMAT_DEPTH_S16);
+
   switch (filter) {
-    case SCHRO_WAVELET_DESL_9_3:
-      schro_iwt_desl_9_3 (data, stride, width, height, tmp);
+    case SCHRO_WAVELET_DESLAURIES_DUBUC_9_7:
+      schro_iwt_desl_9_3 (fd->data, fd->stride, fd->width, fd->height, tmp);
       break;
-    case SCHRO_WAVELET_5_3:
-      schro_iwt_5_3 (data, stride, width, height, tmp);
+    case SCHRO_WAVELET_LE_GALL_5_3:
+      schro_iwt_5_3 (fd->data, fd->stride, fd->width, fd->height, tmp);
       break;
-    case SCHRO_WAVELET_13_5:
-      schro_iwt_13_5 (data, stride, width, height, tmp);
+    case SCHRO_WAVELET_DESLAURIES_DUBUC_13_7:
+      schro_iwt_13_5 (fd->data, fd->stride, fd->width, fd->height, tmp);
       break;
     case SCHRO_WAVELET_HAAR_0:
-      schro_iwt_haar0 (data, stride, width, height, tmp);
+      schro_iwt_haar0 (fd->data, fd->stride, fd->width, fd->height, tmp);
       break;
     case SCHRO_WAVELET_HAAR_1:
-      schro_iwt_haar1 (data, stride, width, height, tmp);
-      break;
-    case SCHRO_WAVELET_HAAR_2:
-      schro_iwt_haar2 (data, stride, width, height, tmp);
+      schro_iwt_haar1 (fd->data, fd->stride, fd->width, fd->height, tmp);
       break;
     case SCHRO_WAVELET_FIDELITY:
-      schro_iwt_fidelity (data, stride, width, height, tmp);
+      schro_iwt_fidelity (fd->data, fd->stride, fd->width, fd->height, tmp);
       break;
-    case SCHRO_WAVELET_DAUB_9_7:
-      schro_iwt_daub_9_7(data, stride, width, height, tmp);
+    case SCHRO_WAVELET_DAUBECHIES_9_7:
+      schro_iwt_daub_9_7(fd->data, fd->stride, fd->width, fd->height, tmp);
       break;
   }
 }
@@ -715,23 +752,38 @@ void schro_iiwt_desl_9_3 (int16_t *data, int stride, int width, int height,
 
 #define ROW(row) ((int16_t *)OFFSET(data, (row)*stride))
 
+  /* FIXME */
+  SCHRO_ASSERT(height>=6);
+
   for(i=-6;i<height;i++){
     int i1 = i+2;
     int i2 = i+6;
     if ((i2&1) == 0 && i2>=0 && i2 < height) {
-      static const int16_t stage2_offset_shift[] = { 1, 2 };
+      static const int16_t stage2_offset_shift[] = { 2, 2 };
       if (i2 == 0) {
-        static const int16_t stage2_weights[] = { -1, -1 };
+        //static const int16_t stage2_weights[] = { -1, -1 };
+        oil_add2_rshift_sub_s16 (
+            OFFSET(data,i2*stride), OFFSET(data, i2*stride),
+            OFFSET(data, (i2+1)*stride), OFFSET(data, (i2+1)*stride),
+            stage2_offset_shift, width);
+#if 0
         oil_mas2_across_add_s16 (
             OFFSET(data,i2*stride), OFFSET(data, i2*stride),
             OFFSET(data, (i2+1)*stride), OFFSET(data, (i2+1)*stride),
             stage2_weights, stage2_offset_shift, width);
+#endif
       } else {
-        static const int16_t stage2_weights[] = { -1, -1 };
+        //static const int16_t stage2_weights[] = { -1, -1 };
+        oil_add2_rshift_sub_s16 (
+            OFFSET(data,i2*stride), OFFSET(data, i2*stride),
+            OFFSET(data, (i2-1)*stride), OFFSET(data, (i2+1)*stride),
+            stage2_offset_shift, width);
+#if 0
         oil_mas2_across_add_s16 (
             OFFSET(data,i2*stride), OFFSET(data, i2*stride),
             OFFSET(data, (i2-1)*stride), OFFSET(data, (i2+1)*stride),
             stage2_weights, stage2_offset_shift, width);
+#endif
       }
     }
     if ((i1&1) == 0 && i1>=0 && i1 < height) {
@@ -778,10 +830,10 @@ void schro_iiwt_desl_9_3 (int16_t *data, int stride, int width, int height,
 void schro_iiwt_5_3 (int16_t *data, int stride, int width, int height,
     int16_t *tmp)
 {
-  static const int16_t stage1_weights[] = { 1, 1 };
-  static const int16_t stage2_weights[] = { -1, -1 };
+  //static const int16_t stage1_weights[] = { 1, 1 };
+  //static const int16_t stage2_weights[] = { -1, -1 };
   static const int16_t stage1_offset_shift[] = { 1, 1 };
-  static const int16_t stage2_offset_shift[] = { 1, 2 };
+  static const int16_t stage2_offset_shift[] = { 2, 2 };
   int i;
 
 #define ROW(row) ((int16_t *)OFFSET(data, (row)*stride))
@@ -796,12 +848,20 @@ void schro_iiwt_5_3 (int16_t *data, int stride, int width, int height,
       } else {
         d = OFFSET(data, 1*stride);
       }
+      oil_add2_rshift_sub_s16 (
+          OFFSET(data, i2*stride),
+          OFFSET(data, i2*stride),
+          d,
+          OFFSET(data, (i2+1)*stride),
+          stage2_offset_shift, width);
+#if 0
       oil_mas2_across_add_s16 (
           OFFSET(data, i2*stride),
           OFFSET(data, i2*stride),
           d,
           OFFSET(data, (i2+1)*stride),
           stage2_weights, stage2_offset_shift, width);
+#endif
     }
     if ((i1&1) == 0 && i1 >= 0 && i1 < height) {
       int16_t *d;
@@ -810,12 +870,20 @@ void schro_iiwt_5_3 (int16_t *data, int stride, int width, int height,
       } else {
         d = OFFSET(data,(height-2)*stride);
       }
+      oil_add2_rshift_add_s16 (
+          OFFSET(data, (i1+1)*stride),
+          OFFSET(data, (i1+1)*stride),
+          OFFSET(data, i1*stride),
+          d,
+          stage1_offset_shift, width);
+#if 0
       oil_mas2_across_add_s16 (
           OFFSET(data, (i1+1)*stride),
           OFFSET(data, (i1+1)*stride),
           OFFSET(data, i1*stride),
           d,
           stage1_weights, stage1_offset_shift, width);
+#endif
     } 
     if (i >=0 && i < height) {
       int16_t *hi = tmp + 2;
@@ -836,6 +904,9 @@ void schro_iiwt_13_5 (int16_t *data, int stride, int width, int height,
 {
   int i;
 
+  /* FIXME */
+  SCHRO_ASSERT(height>=6);
+
 #define ROW(row) ((int16_t *)OFFSET(data, (row)*stride))
   for(i=-8;i<height;i++){
     int i1 = i+4;
@@ -952,16 +1023,14 @@ void schro_iiwt_haar1 (int16_t *data, int stride, int width, int height, int16_t
   schro_iiwt_haar (data, stride, width, height, tmp, 1);
 }
 
-void schro_iiwt_haar2 (int16_t *data, int stride, int width, int height, int16_t *tmp)
-{
-  schro_iiwt_haar (data, stride, width, height, tmp, 2);
-}
-
 void schro_iiwt_fidelity (int16_t *data, int stride, int width, int height,
     int16_t *tmp)
 {
   int i;
 
+  /* FIXME */
+  SCHRO_ASSERT(height>=16);
+
 #define ROW(row) ((int16_t *)OFFSET(data, (row)*stride))
   for(i=-16;i<height;i++){
     int i1 = i+8;
@@ -1116,33 +1185,33 @@ void schro_iiwt_daub_9_7 (int16_t *data, int stride, int width, int height,
 }
 
 void
-schro_wavelet_inverse_transform_2d (int filter, int16_t *data, int stride,
-    int width, int height, int16_t *tmp)
+schro_wavelet_inverse_transform_2d (SchroFrameData *fd, int filter,
+    int16_t *tmp)
 {
+  SCHRO_ASSERT(SCHRO_FRAME_FORMAT_DEPTH(fd->format) ==
+      SCHRO_FRAME_FORMAT_DEPTH_S16);
+
   switch (filter) {
-    case SCHRO_WAVELET_DESL_9_3:
-      schro_iiwt_desl_9_3 (data, stride, width, height, tmp);
+    case SCHRO_WAVELET_DESLAURIES_DUBUC_9_7:
+      schro_iiwt_desl_9_3 (fd->data, fd->stride, fd->width, fd->height, tmp);
       break;
-    case SCHRO_WAVELET_5_3:
-      schro_iiwt_5_3 (data, stride, width, height, tmp);
+    case SCHRO_WAVELET_LE_GALL_5_3:
+      schro_iiwt_5_3 (fd->data, fd->stride, fd->width, fd->height, tmp);
       break;
-    case SCHRO_WAVELET_13_5:
-      schro_iiwt_13_5 (data, stride, width, height, tmp);
+    case SCHRO_WAVELET_DESLAURIES_DUBUC_13_7:
+      schro_iiwt_13_5 (fd->data, fd->stride, fd->width, fd->height, tmp);
       break;
     case SCHRO_WAVELET_HAAR_0:
-      schro_iiwt_haar0 (data, stride, width, height, tmp);
+      schro_iiwt_haar0 (fd->data, fd->stride, fd->width, fd->height, tmp);
       break;
     case SCHRO_WAVELET_HAAR_1:
-      schro_iiwt_haar1 (data, stride, width, height, tmp);
-      break;
-    case SCHRO_WAVELET_HAAR_2:
-      schro_iiwt_haar2 (data, stride, width, height, tmp);
+      schro_iiwt_haar1 (fd->data, fd->stride, fd->width, fd->height, tmp);
       break;
     case SCHRO_WAVELET_FIDELITY:
-      schro_iiwt_fidelity (data, stride, width, height, tmp);
+      schro_iiwt_fidelity (fd->data, fd->stride, fd->width, fd->height, tmp);
       break;
-    case SCHRO_WAVELET_DAUB_9_7:
-      schro_iiwt_daub_9_7(data, stride, width, height, tmp);
+    case SCHRO_WAVELET_DAUBECHIES_9_7:
+      schro_iiwt_daub_9_7(fd->data, fd->stride, fd->width, fd->height, tmp);
       break;
   }
 }
