@@ -26,13 +26,12 @@ main (int argc, char *argv[])
   }
 
   schro_arith_decode_init (a);
-  schro_arith_init_contexts (a);
 
   for(i=0;i<10000;i++){
     printf("hi=%d lo=%d code=%04x count0=%d count1=%d\n",
         a->high, a->low, a->code, a->contexts[0].count0,
         a->contexts[0].count1);
-    value = schro_arith_context_binary_decode (a, 0);
+    value = schro_arith_binary_decode (a, 0);
     printf(" --> %d\n", value);
   }
 
@@ -62,11 +61,10 @@ main (int argc, char *argv[])
 
   output_buffer = schro_buffer_new_and_alloc (BUFFER_SIZE);
   schro_arith_encode_init (a, output_buffer);
-  schro_arith_init_contexts (a);
 
   for(i=0;i<n_bytes;i++){
     for(j=0;j<8;j++){
-      schro_arith_context_encode_bit (a, j, (input_buffer[i]>>(7-j))&1);
+      schro_arith_encode_bit (a, j, (input_buffer[i]>>(7-j))&1);
     }
   }
   schro_arith_flush (a);
