@@ -16,19 +16,19 @@ int16_t tmp[100];
 int16_t tmp2[100];
 int16_t *frame_data;
 
-int filtershift[] = { 1, 1, 1, 0, 1, 2, 0, 1 };
+int filtershift[] = { 1, 1, 1, 0, 1, 0, 1 };
 
-void synth(int16_t *a, int filter, int n);
-void split (int16_t *a, int filter, int n);
-void synth_schro_ext (int16_t *a, int filter, int n);
-void split_schro_ext (int16_t *a, int n, int filter);
-void deinterleave (int16_t *a, int n);
-void interleave (int16_t *a, int n);
-void dump (int16_t *a, int n);
-void dump_cmp (int16_t *a, int16_t *b, int n);
+static void synth(int16_t *a, int filter, int n);
+static void split (int16_t *a, int filter, int n);
+static void synth_schro_ext (int16_t *a, int filter, int n);
+static void split_schro_ext (int16_t *a, int n, int filter);
+static void deinterleave (int16_t *a, int n);
+static void interleave (int16_t *a, int n);
+static void dump (int16_t *a, int n);
+static void dump_cmp (int16_t *a, int16_t *b, int n);
 
 
-void
+static void
 gen_const(int16_t *a, int n)
 {
   int i;
@@ -38,7 +38,7 @@ gen_const(int16_t *a, int n)
   }
 }
 
-void
+static void
 gen_ramp(int16_t *a, int n)
 {
   int i;
@@ -48,7 +48,7 @@ gen_ramp(int16_t *a, int n)
   }
 }
 
-void
+static void
 gen_alternating(int16_t *a, int n)
 {
   int i;
@@ -58,7 +58,7 @@ gen_alternating(int16_t *a, int n)
   }
 }
 
-void
+static void
 gen_spike(int16_t *a, int n)
 {
   int i;
@@ -68,7 +68,7 @@ gen_spike(int16_t *a, int n)
   }
 }
 
-void
+static void
 gen_edge(int16_t *a, int n)
 {
   int i;
@@ -78,7 +78,7 @@ gen_edge(int16_t *a, int n)
   }
 }
 
-void
+static void
 gen_random(int16_t *a, int n)
 {
   int i;
@@ -103,16 +103,16 @@ Generator generators[] = {
   { "random", gen_random }
 };
 
-void
+static void
 local_test(int filter)
 {
   int16_t *a = tmp + 10;
   int n = 20;
-  int $;
+  int i;
 
-  for($=0;$<sizeof(generators)/sizeof(generators[0]);$++){
-    printf("  test \"%s\":\n", generators[$].name);
-    generators[$].create(a,n);
+  for(i=0;i<sizeof(generators)/sizeof(generators[0]);i++){
+    printf("  test \"%s\":\n", generators[i].name);
+    generators[i].create(a,n);
     dump(a,n);
     split(a,n,filter);
     deinterleave(a,n);
@@ -131,23 +131,23 @@ local_test(int filter)
   printf("\n");
 }
 
-void
+static void
 random_test(int filter)
 {
   int16_t *a = tmp + 10;
   int n = 20;
-  int $;
+  int i;
   int16_t b[100];
   int16_t c[100];
   int failed = 0;
 
   printf("  testing random arrays (split):\n");
-  for($=0;$<100;$++){
+  for(i=0;i<100;i++){
 #if 0
     {
       int i;
       for(i=0;i<n;i++){
-        a[i] = (i==$)*100;
+        a[i] = (i==i)*100;
       }
     }
 #endif
@@ -175,7 +175,7 @@ random_test(int filter)
   printf("\n");
 
   printf("  testing random arrays (synth):\n");
-  for($=0;$<100;$++){
+  for(i=0;i<100;i++){
     gen_random(a,n);
     memcpy(b,a,n*sizeof(int16_t));
 
@@ -206,7 +206,7 @@ main (int argc, char *argv[])
 
   schro_init();
 
-  for(filter=0;filter<=SCHRO_WAVELET_DAUB_9_7;filter++){
+  for(filter=0;filter<=SCHRO_WAVELET_DAUBECHIES_9_7;filter++){
     printf("Filter %d:\n", filter);
     local_test(filter);
     random_test(filter);
@@ -215,7 +215,7 @@ main (int argc, char *argv[])
   return 0;
 }
 
-void
+static void
 dump (int16_t *a, int n)
 {
   int i;
@@ -225,7 +225,7 @@ dump (int16_t *a, int n)
   printf("\n");
 }
 
-void
+static void
 dump_cmp (int16_t *a, int16_t *b, int n)
 {
   int i;
@@ -239,7 +239,7 @@ dump_cmp (int16_t *a, int16_t *b, int n)
   printf("\n");
 }
 
-void
+static void
 interleave (int16_t *a, int n)
 {
   int i;
@@ -252,7 +252,7 @@ interleave (int16_t *a, int n)
   }
 }
 
-void
+static void
 deinterleave (int16_t *a, int n)
 {
   int i;
@@ -265,7 +265,7 @@ deinterleave (int16_t *a, int n)
   }
 }
 
-void
+static void
 extend(int16_t *a, int n)
 {
   a[-8] = a[0];
@@ -286,13 +286,13 @@ extend(int16_t *a, int n)
   a[n+7] = a[n-1];
 }
 
-void
+static void
 synth(int16_t *a, int n, int filter)
 {
   int i;
 
   switch (filter) {
-    case SCHRO_WAVELET_DESL_9_3:
+    case SCHRO_WAVELET_DESLAURIES_DUBUC_9_7:
       extend(a,n);
       for(i=0;i<n;i+=2){
         a[i] -= (a[i-1] + a[i+1] + 2)>>2;
@@ -302,7 +302,7 @@ synth(int16_t *a, int n, int filter)
         a[i + 1] += (-a[i-2] + 9 * a[i] + 9 * a[i+2] - a[i+4] + 8)>>4;
       }
       break;
-    case SCHRO_WAVELET_5_3:
+    case SCHRO_WAVELET_LE_GALL_5_3:
       extend(a,n);
       for(i=0;i<n;i+=2){
         a[i] -= (a[i-1] + a[i+1] + 2)>>2;
@@ -312,7 +312,7 @@ synth(int16_t *a, int n, int filter)
         a[i + 1] += (a[i] + a[i+2] + 1)>>1;
       }
       break;
-    case SCHRO_WAVELET_13_5:
+    case SCHRO_WAVELET_DESLAURIES_DUBUC_13_7:
       extend(a,n);
       for(i=0;i<n;i+=2){
         a[i] -= (-a[i-3] + 9 * a[i-1] + 9 * a[i+1] - a[i+3] + 16)>>5;
@@ -324,7 +324,6 @@ synth(int16_t *a, int n, int filter)
       break;
     case SCHRO_WAVELET_HAAR_0:
     case SCHRO_WAVELET_HAAR_1:
-    case SCHRO_WAVELET_HAAR_2:
       for(i=0;i<n;i+=2){
         a[i] -= (a[i+1] + 1)>>1;
       }
@@ -344,7 +343,7 @@ synth(int16_t *a, int n, int filter)
             161*a[i+1] - 46*a[i+3] + 21*a[i+5] - 8*a[i+7] + 128) >> 8;
       }
       break;
-    case SCHRO_WAVELET_DAUB_9_7:
+    case SCHRO_WAVELET_DAUBECHIES_9_7:
       extend(a,n);
       for(i=0;i<n;i+=2){
         a[i] -= (1817*a[i-1] + 1817 * a[i+1] + 2048)>>12;
@@ -365,13 +364,13 @@ synth(int16_t *a, int n, int filter)
   }
 }
 
-void
+static void
 split (int16_t *a, int n, int filter)
 {
   int i;
 
   switch (filter) {
-    case SCHRO_WAVELET_DESL_9_3:
+    case SCHRO_WAVELET_DESLAURIES_DUBUC_9_7:
       extend(a,n);
       for(i=0;i<n;i+=2){
         a[i + 1] -= (-a[i-2] + 9 * a[i] + 9 * a[i+2] - a[i+4] + 8)>>4;
@@ -381,7 +380,7 @@ split (int16_t *a, int n, int filter)
         a[i] += (a[i-1] + a[i+1] + 2)>>2;
       }
       break;
-    case SCHRO_WAVELET_5_3:
+    case SCHRO_WAVELET_LE_GALL_5_3:
       extend(a,n);
       for(i=0;i<n;i+=2){
         a[i + 1] -= (a[i] + a[i+2] + 1)>>1;
@@ -391,7 +390,7 @@ split (int16_t *a, int n, int filter)
         a[i] += (a[i-1] + a[i+1] + 2)>>2;
       }
       break;
-    case SCHRO_WAVELET_13_5:
+    case SCHRO_WAVELET_DESLAURIES_DUBUC_13_7:
       extend(a,n);
       for(i=0;i<n;i+=2){
         a[i+1] -= (-a[i-2] + 9 * a[i] + 9 * a[i+2] - a[i+4] + 8)>>4;
@@ -403,7 +402,6 @@ split (int16_t *a, int n, int filter)
       break;
     case SCHRO_WAVELET_HAAR_0:
     case SCHRO_WAVELET_HAAR_1:
-    case SCHRO_WAVELET_HAAR_2:
       for(i=0;i<n;i+=2){
         a[i+1] -= a[i];
       }
@@ -423,7 +421,7 @@ split (int16_t *a, int n, int filter)
             81*a[i+2] - 25*a[i+4] + 10*a[i+6] - 2*a[i+8] + 128) >> 8;
       }
       break;
-    case SCHRO_WAVELET_DAUB_9_7:
+    case SCHRO_WAVELET_DAUBECHIES_9_7:
       extend(a,n);
       for(i=0;i<n;i+=2){
         a[i+1] -= (6497*a[i] + 6497 * a[i+2] + 2048)>>12;
@@ -444,7 +442,7 @@ split (int16_t *a, int n, int filter)
   }
 }
 
-void
+static void
 split_schro_ext (int16_t *a, int n, int filter)
 {
   int16_t tmp1[100], *hi;
@@ -456,24 +454,23 @@ split_schro_ext (int16_t *a, int n, int filter)
   oil_deinterleave2_s16 (hi, lo, a, n/2);
 
   switch (filter) {
-    case SCHRO_WAVELET_DESL_9_3:
+    case SCHRO_WAVELET_DESLAURIES_DUBUC_9_7:
       schro_split_ext_desl93 (hi, lo, n/2);
       break;
-    case SCHRO_WAVELET_5_3:
+    case SCHRO_WAVELET_LE_GALL_5_3:
       schro_split_ext_53 (hi, lo, n/2);
       break;
-    case SCHRO_WAVELET_13_5:
+    case SCHRO_WAVELET_DESLAURIES_DUBUC_13_7:
       schro_split_ext_135 (hi, lo, n/2);
       break;
     case SCHRO_WAVELET_HAAR_0:
     case SCHRO_WAVELET_HAAR_1:
-    case SCHRO_WAVELET_HAAR_2:
       schro_split_ext_haar (hi, lo, n/2);
       break;
     case SCHRO_WAVELET_FIDELITY:
       schro_split_ext_fidelity (hi, lo, n/2);
       break;
-    case SCHRO_WAVELET_DAUB_9_7:
+    case SCHRO_WAVELET_DAUBECHIES_9_7:
       schro_split_ext_daub97(hi, lo, n/2);
       break;
   }
@@ -481,7 +478,7 @@ split_schro_ext (int16_t *a, int n, int filter)
 
 }
 
-void
+static void
 synth_schro_ext (int16_t *a, int n, int filter)
 {
   int16_t tmp1[100], *hi;
@@ -493,24 +490,23 @@ synth_schro_ext (int16_t *a, int n, int filter)
   oil_deinterleave2_s16 (hi, lo, a, n/2);
 
   switch (filter) {
-    case SCHRO_WAVELET_DESL_9_3:
+    case SCHRO_WAVELET_DESLAURIES_DUBUC_9_7:
       schro_synth_ext_desl93 (hi, lo, n/2);
       break;
-    case SCHRO_WAVELET_5_3:
+    case SCHRO_WAVELET_LE_GALL_5_3:
       schro_synth_ext_53 (hi, lo, n/2);
       break;
-    case SCHRO_WAVELET_13_5:
+    case SCHRO_WAVELET_DESLAURIES_DUBUC_13_7:
       schro_synth_ext_135 (hi, lo, n/2);
       break;
     case SCHRO_WAVELET_HAAR_0:
     case SCHRO_WAVELET_HAAR_1:
-    case SCHRO_WAVELET_HAAR_2:
       schro_synth_ext_haar (hi, lo, n/2);
       break;
     case SCHRO_WAVELET_FIDELITY:
       schro_synth_ext_fidelity (hi, lo, n/2);
       break;
-    case SCHRO_WAVELET_DAUB_9_7:
+    case SCHRO_WAVELET_DAUBECHIES_9_7:
       schro_synth_ext_daub97(hi, lo, n/2);
       break;
   }
@@ -522,7 +518,8 @@ synth_schro_ext (int16_t *a, int n, int filter)
 
 
 
-void
+#ifdef unused
+static void
 schro_split_desl93 (int16_t *hi, int16_t *lo, int n)
 {
   static int16_t stage1_weights[] = { 1, -9, -9, 1 };
@@ -550,7 +547,7 @@ schro_split_desl93 (int16_t *hi, int16_t *lo, int n)
   oil_mas2_add_s16 (hi + 1, hi + 1, lo, stage2_weights, stage2_offset_shift, n - 1);
 }
 
-void
+static void
 schro_split_53 (int16_t *hi, int16_t *lo, int n)
 {
   static int16_t stage1_weights[] = { -1, -1 };
@@ -576,7 +573,7 @@ schro_split_53 (int16_t *hi, int16_t *lo, int n)
   oil_mas2_add_s16 (hi + 1, hi + 1, lo, stage2_weights, stage2_offset_shift, n - 1);
 }
 
-void
+static void
 schro_split_135 (int16_t *hi, int16_t *lo, int n)
 {
   static int16_t stage1_weights[] = { 1, -9, -9, 1 };
@@ -606,7 +603,7 @@ schro_split_135 (int16_t *hi, int16_t *lo, int n)
   oil_mas4_add_s16 (hi + 2, hi + 2, lo, stage2_weights, stage2_offset_shift, n - 3);
 }
 
-void
+static void
 schro_split_haar (int16_t *hi, int16_t *lo, int n)
 {
   int i;
@@ -619,7 +616,7 @@ schro_split_haar (int16_t *hi, int16_t *lo, int n)
   }
 }
 
-void
+static void
 schro_split_fidelity (int16_t *hi, int16_t *lo, int n)
 {
   static int16_t stage1_weights[] = { -8, 21, -46, 161, 161, -46, 21, -8 };
@@ -675,7 +672,7 @@ schro_split_fidelity (int16_t *hi, int16_t *lo, int n)
   oil_mas8_add_s16 (lo + 3, lo + 3, hi, stage2_weights, stage2_offset_shift, n - 7);
 }
 
-void
+static void
 schro_split_daub97 (int16_t *hi, int16_t *lo, int n)
 {
   static int16_t stage1_weights[] = { -6497, -6497 };
@@ -731,24 +728,23 @@ split_schro (int16_t *a, int n, int filter)
   oil_deinterleave2_s16 (hi, lo, a, n/2);
 
   switch (filter) {
-    case SCHRO_WAVELET_DESL_9_3:
+    case SCHRO_WAVELET_DESLAURIES_DUBUC_9_7:
       schro_split_desl93 (hi, lo, n/2);
       break;
-    case SCHRO_WAVELET_5_3:
+    case SCHRO_WAVELET_LE_GALL_5_3:
       schro_split_53 (hi, lo, n/2);
       break;
-    case SCHRO_WAVELET_13_5:
+    case SCHRO_WAVELET_DESLAURIES_DUBUC_13_7:
       schro_split_135 (hi, lo, n/2);
       break;
     case SCHRO_WAVELET_HAAR_0:
     case SCHRO_WAVELET_HAAR_1:
-    case SCHRO_WAVELET_HAAR_2:
       schro_split_haar (hi, lo, n/2);
       break;
     case SCHRO_WAVELET_FIDELITY:
       schro_split_fidelity (hi, lo, n/2);
       break;
-    case SCHRO_WAVELET_DAUB_9_7:
+    case SCHRO_WAVELET_DAUBECHIES_9_7:
       schro_split_daub97(hi, lo, n/2);
       break;
   }
@@ -757,7 +753,7 @@ split_schro (int16_t *a, int n, int filter)
 }
 #endif
 
-void
+static void
 schro_synth_desl93 (int16_t *hi, int16_t *lo, int n)
 {
   static int16_t stage1_weights[] = { -1, -1 };
@@ -791,7 +787,7 @@ schro_synth_desl93 (int16_t *hi, int16_t *lo, int n)
   oil_mas4_add_s16 (lo, lo, hi - 1, stage2_weights, stage2_offset_shift, n);
 }
 
-void
+static void
 schro_synth_53 (int16_t *hi, int16_t *lo, int n)
 {
   static int16_t stage1_weights[] = { -1, -1 };
@@ -821,7 +817,7 @@ schro_synth_53 (int16_t *hi, int16_t *lo, int n)
   oil_mas2_add_s16 (lo, lo, hi, stage2_weights, stage2_offset_shift, n);
 }
 
-void
+static void
 schro_synth_135 (int16_t *hi, int16_t *lo, int n)
 {
   static int16_t stage1_weights[] = { 1, -9, -9, 1 };
@@ -851,7 +847,7 @@ schro_synth_135 (int16_t *hi, int16_t *lo, int n)
   oil_mas4_add_s16 (lo, lo, hi-1, stage2_weights, stage2_offset_shift, n);
 }
 
-void
+static void
 schro_synth_haar (int16_t *hi, int16_t *lo, int n)
 {
   int i;
@@ -873,7 +869,7 @@ schro_synth_haar (int16_t *hi, int16_t *lo, int n)
   }
 }
 
-void
+static void
 schro_synth_fidelity (int16_t *hi, int16_t *lo, int n)
 {
   static int16_t stage1_weights[] = { -2, 10, -25, 81, 81, -25, 10, -2 };
@@ -915,7 +911,7 @@ schro_synth_fidelity (int16_t *hi, int16_t *lo, int n)
   oil_mas8_add_s16 (hi, hi, lo - 4, stage2_weights, stage2_offset_shift, n);
 }
 
-void
+static void
 schro_synth_daub97 (int16_t *hi, int16_t *lo, int n)
 {
   static int16_t stage1_weights[] = { -1817, -1817 };
@@ -965,7 +961,7 @@ schro_synth_daub97 (int16_t *hi, int16_t *lo, int n)
   oil_mas2_add_s16 (lo, lo, hi, stage4_weights, stage34_offset_shift, n);
 }
 
-void
+static void
 synth_schro (int16_t *a, int n, int filter)
 {
   int16_t tmp1[100], *hi;
@@ -977,28 +973,28 @@ synth_schro (int16_t *a, int n, int filter)
   oil_deinterleave2_s16 (hi, lo, a, n/2);
 
   switch (filter) {
-    case SCHRO_WAVELET_DESL_9_3:
+    case SCHRO_WAVELET_DESLAURIES_DUBUC_9_7:
       schro_synth_desl93 (hi, lo, n/2);
       break;
-    case SCHRO_WAVELET_5_3:
+    case SCHRO_WAVELET_LE_GALL_5_3:
       schro_synth_53 (hi, lo, n/2);
       break;
-    case SCHRO_WAVELET_13_5:
+    case SCHRO_WAVELET_DESLAURIES_DUBUC_13_7:
       schro_synth_135 (hi, lo, n/2);
       break;
     case SCHRO_WAVELET_HAAR_0:
     case SCHRO_WAVELET_HAAR_1:
-    case SCHRO_WAVELET_HAAR_2:
       schro_synth_haar (hi, lo, n/2);
       break;
     case SCHRO_WAVELET_FIDELITY:
       schro_synth_fidelity (hi, lo, n/2);
       break;
-    case SCHRO_WAVELET_DAUB_9_7:
+    case SCHRO_WAVELET_DAUBECHIES_9_7:
       schro_synth_daub97(hi, lo, n/2);
       break;
   }
 
   oil_interleave2_s16 (a, hi, lo, n/2);
 }
+#endif
 
